;****************************************************************************************
;*                           SAEGEZAHN VON DL3UKH                                       *
;*                           ====================                                       *
;*                                                                                      *
;*--------------------------------------------------------------------------------------*
;*                                                                                      *
;* ALS PIC-IC WURDE GENOMMEN PIC16F88                                                   *
;* ALS OSZILLATOR WURDE EIN QUARZ MIT 20,00MHZ GENOMMEN. (T=0.05SEC ALSO DIE TAKTZEIT) *
;*                                   EIN BEFEHL HAT MEIST 4 TAKTE SIND 0,2SEC          *
;*                                                                                      *
;*--------------------------------------------------------------------------------------*
;*                           BEGINN: 03.12.16                                           *
;*                           ENDE  :                                            *
;*                         GETESTET:                                            *
;****************************************************************************************

        LIST    P=16F88       
        TITLE   "SAEGEZAHN"

        ERRORLEVEL -302
        INCLUDE <P16F88.INC>

__CONFIG _CONFIG1, _CP_OFF & _CCPMX_RB0 & _DEBUG_OFF & _WRT_OFF & _CPD_OFF & _LVP_OFF & _BOREN_OFF & _MCLRE_OFF & _PWRTE_OFF & _WDTE_OFF & _HS_OSC
__CONFIG _CONFIG2, _IESO_OFF & _FCMEN_OFF



;--------------------------------------------------------------------------
;REGISTER ZUORDNEN NACH DEM DIE FESTEN-REGISTER ZU ENDE SIND
;ES GEHT ALSO ERST AB 22 HEX LOS
;DIES IST ALSO DIE RAM-SPEICHER ZUORDNUNG


HELP                    EQU     0X22        ;HILFSZELLE
ZAE                     EQU     0X23        ;HILFSZELLE
BYTE                    EQU     0X24        ;BYTE AUSGEBEN BEIM ZAEHLEN
AD_L                    EQU     0X25
AD_H                    EQU     0X26

W_STACK                 EQU     0X27
S_STACK                 EQU     0X28

;--------------------------- VARIABLEN -------------------------------------
BIT7                    EQU     7       ; BIT 7
BIT6                    EQU     6       ; BIT 6
BIT5                    EQU     5       ; BIT 5
BIT4                    EQU     4       ; BIT 4
BIT3                    EQU     3       ; BIT 3 
BIT2                    EQU     2       ; BIT 2
BIT1                    EQU     1       ; BIT 1 
BIT0                    EQU     0       ; BIT 0
;--------------------------------------------------------------------------

;------------------------- MACROS -----------------------------------------

BANK0           MACRO                   ;MACRO TO SELECT DATA RAM BANK 0
                BCF     STATUS,RP0
                BCF     STATUS,RP1
                ENDM

BANK1           MACRO                   ;MACRO TO SELECT DATA RAM BANK 1
                BSF     STATUS,RP0
                BCF     STATUS,RP1
                ENDM

BANK2           MACRO                   ;MACRO TO SELECT DATA RAM BANK 2
                BCF     STATUS,RP0
                BSF     STATUS,RP1
                ENDM

BANK3           MACRO                   ;MACRO TO SELECT DATA RAM BANK 3
                BSF     STATUS,RP0
                BSF     STATUS,RP1
                ENDM

PUSH    MACRO
        MOVWF   W_STACK
        MOVF    STATUS, W
        MOVWF   S_STACK
        ENDM

POP     MACRO
        MOVF    S_STACK, W
        MOVWF   STATUS
        SWAPF   W_STACK, F
        SWAPF   W_STACK, W
        ENDM

;----------------------------------------------------------------------------
;-----------------------------------------------------------------------------

;SPEICHERANFANG BZW. BEGINN

                ORG     0X0000
                GOTO    MAIN               ; SPRUNG ZUM HAUPTPROGRAMM
                ORG     0X0004
                GOTO    INTERRUPT

                ORG 0X0010

;*****************************************************************************
;*                            HAUPTPROGRAMM                                  *
;*                              01.02.11                                     *
;*****************************************************************************
;PORT-INITIALISIERUNG
MAIN            CLRF    PORTA                           
                CLRF    PORTB

                BANK1
                MOVLW   B'00001000'
                MOVWF   OSCCON

                MOVLW   0X07                    ;COMPARATORS ALLE AUS VOM PORT A
                MOVWF   CMCON

                MOVLW   B'11111111'             ;INIT INPUT/OUTPUT AM PORTA ALLES EINGANG / BEI 1 EINGANG  6+7 SIND QUARZ
                MOVWF   TRISA
                MOVLW   B'00000000'             ;INIT I/O PORTB  ALLES AUSGANG 
                MOVWF   TRISB

;INIT TIMER 0 FUER 1MSEC
                MOVLW   B'10000111'             ;BIT0-2 1=PRESCALER256, BIT3 0=TMR0, BIT4 0=ENTFAELLT, BIT5 0=TAKT VOM SYSTEM, BIT6 0=KEIN RB-INTERRUPT, BIT7 1=PULL-UPS DISABLE 
                MOVWF   OPTION_REG

;AD-INIT
                MOVLW   B'00000010'  ;RA1 ALS AD1
                MOVWF   ANSEL
;AD-INIT
                MOVLW   B'10000000'  ;RECHTSBNDIG B1 / FOSZ=32 B0 / Vss+Vdd REF-SPANNUNG B00
                MOVWF   ADCON1

                BSF     PIE1, ADIE   ;INTERRUPT-ENABLE BEI AD

                BANK0
;AD-INIT
                MOVLW   B'10001001'  ;FOSZ=32 B10 / AD1 B001 / GO-IST AUS + ON IST EIN B01
                MOVWF   ADCON0

                BCF    PIR1, ADIF 

;INTERRUPT ZULASSEN
                BSF     INTCON, GIE         ;ALLGEMEINES INTERRUPT ENABLE 
                BSF     INTCON, PEIE        ;INTERUPT ZULASSEN PEREPHERIE 

;100MSEC ZEITSCHLEIFE
                MOVLW   D'100'                  ;ZEITSCHLEIFE LADEN IN W MIT DEZIMAL 100 ALSO---> 100*1MSEC=100MSEC
                CALL    ZS1MSEC                 ;ZEITSCHLEIFE ABARBEITEN
;---------------------------------------------------------------------------------------------------------------------

ANF             BSF     ADCON0, GO_DONE     ;GEHT LOS

                MOVLW   0XFF
                MOVWF   BYTE

;----------------------------------------
AUSGEBEN        MOVF    BYTE, W     ;T=4   
                MOVWF   PORTB       ;T=4

                MOVF    AD_L, W     ;T=4
                MOVWF   ZAE         ;T=4 GES.:T=16

ZS1             DECFSZ  ZAE, F      ;T=4 / 8VERNACHLSSIGT
                GOTO    ZS1         ;T=8

;MAX. FREQUENZ ZAE=1
;1*12T=12T   +    12T+16T =40T
;40*0,05SEC=2SEC * 256 =512SEC
;F=1:T  1:512SEC=1953HZ

;MIN. FREQUENZ ZAE=64
;64*12T=768T   +    12T+16T =796T
;796*0,05SEC=39,8SEC * 256 =10188SEC
;F=1:T  1:10188SEC=98,15HZ

                DECFSZ  BYTE, F     ;T=4
                GOTO    AUSGEBEN    ;T=8 GES.:T=12

                GOTO    ANF

;*****************************************************************************
;*                              INTERRUPT                                    *
;*                              14.06.15                                     *
;*****************************************************************************
INTERRUPT       PUSH
                BCF    PIR1, ADIF 
                BCF    ADCON0, GO_DONE
   
;AD-ERGEBNIS ABSPEICHERN IN RAM-ZELLEN AD_H UND AD_L
                MOVF    ADRESH, W
                MOVWF   AD_H
                BANK1                               
                MOVF    ADRESL, W
                BANK0                               
                MOVWF   AD_L

;---------
;1024 ZU 64 DEZ. MACHEN
                BCF     STATUS, C
                RRF     AD_H, F         ;ZU 512MAX.
                RRF     AD_L, F

                BCF     STATUS, C
                RRF     AD_H, F         ;ZU 256MAX.
                RRF     AD_L, F

                BCF     STATUS, C
                RRF     AD_H, F         ;ZU 128MAX.
                RRF     AD_L, F

                BCF     STATUS, C
                RRF     AD_H, F         ;ZU 64MAX.
                RRF     AD_L, F

                MOVLW    0X00
                SUBWF    AD_L, W
                BTFSS    STATUS, Z
                GOTO     EEE

                INCF     AD_L, F      ;UM EINS ERHHEN ALSO 0X01 FR AD_L --->WENN NULL

EEE             POP
                RETFIE


;*****************************************************************************
;*                              1MSEC ZEITSCHLEIFE                           *
;*                              03.01.14                                     *
;*****************************************************************************
ZS1MSEC                 MOVWF   HELP

ZS11                    MOVLW   D'237'
                        MOVWF   TMR0                    ;DER FESTE RAM-SPEICHER (TMR0) WIRD MIT 100 GELADEN
                        BCF     INTCON, TMR0IF            ;OVERFLOW = 0
                
ZS12                    BTFSS   INTCON, TMR0IF            ;WENN DIE ZEIT VON 10MSEC ABGELAUFEN IST, WIRD T0IF HIGH (1) UND 
                        GOTO    ZS12                    ;ES ERFOLGT EIN SPRUNG ZU "DECFSC ZAE, F"..... VORHER ANDERES ABARBEITEN

                        DECFSZ  HELP, F                 ;ES WIRD DER WERT IN "HELP" ABWRTS GEZHLT, BIS ER (0) IST UND
                        GOTO    ZS11                    ;DANN ERFOLGT EIN SPRUNG ZU "RETURN"-ENDE DES UNTERPROGRAMMS

                        RETURN

;ZEITBERECHNUNG FUER 1MS
;(HIER MUSS MAN IN DEM DATENBLATT (DATASHEET) BEI WWW.MICROCHIP.COM (EIN *.PDF-FILE) NACHSEHEN, WIE ALLES FUNKTIONIERT)

;PRESCALER 256X  *  SYSTEM 4X  * TAKTZEIT ====> 256*4*0.05S=51,2SEC /DIE TAKTZEIT ERGIBT SICH AUS DEM INTERNEN OSZILLATOR
;1MS : 51,2S= 19,5  GENOMMEN 19                                      /T = 1 : F  (F)-OSZILLATORFREQUENZ HIER 16MHZ
;ALSO 
;51,2S*D'19'=0,97MSEC
;TMR0 ALSO D'256'-D'19'=D'237' ENTSPRICHT CA. 1MSEC
;237 DEZIMAL OBEN EINTRAGEN



            END





